﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PermutationAlgorithms.PermAlgorithms.Impl
{
    /// <summary>
    /// Lexicographic permutation algorithm (Algorithm L)
    /// from:
    /// Donald E. Knuth - The Art Of Computer Programming Pre-Fascicle 2B.
    /// A Draft Of Section 7.2.1.2: Generating all permutations
    /// </summary>
    class AlgorithmL : AbstractPermAlgo
    {
        public AlgorithmL(int numOfItems, IPermutationVisitor permVisitor) :
            base(numOfItems, permVisitor) { }

        public override void Generate()
        {
            /* trivial case */
            if (ItemCount == 1)
            {
                VisitPermutation();
                return;
            }

            while (true)
            {
                /* L1: Visit */
                VisitPermutation();

                /* L2: Find the largest index j so that items[j] can be increased:
                 *     Equal to: Find the longest suffix that is not increasing. Element at index j is then the element left to this suffix */
                int j = this.ItemCount - 2;
                while (items[j] >= items[j + 1])
                {
                    if (j == 0)
                        return;
                    j--;
                }

                /* L3: Increase items[j]: Find the rightmost element in the suffix that is greater than element at index j */
                int l = this.ItemCount - 1;
                while (items[j] >= items[l])
                {
                    l--;
                }
                /* swap both items */
                Swap(ref items[j], ref items[l]);

                /* At this place the suffix is decreasing!: items[j+1] >= ... >= items[n-1] */
                /* L4: Reverse items[j+1] ... items[n-1]: */
                l = this.ItemCount - 1;
                int k = j + 1;

                while (l > k)
                {
                    Swap(ref items[k], ref items[l]);
                    k++;
                    l--;
                }

            }
        }
    }
}
