﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PermutationAlgorithms.PermAlgorithms.Impl
{
    /// <summary>
    /// Plain changes permutation algorithm (Algorithm P)
    /// from:
    /// Donald E. Knuth - The Art Of Computer Programming Pre-Fascicle 2B.
    /// A Draft Of Section 7.2.1.2: Generating all permutations
    /// </summary>
    class AlgorithmP : AbstractPermAlgo
    {
        private int[] inversionTable;
        private int[] direction;

        public AlgorithmP(int numOfItems, IPermutationVisitor permVisitor) :
            base(numOfItems, permVisitor) { }

        public override void Generate()
        {
            inversionTable = new int[ItemCount];
            direction = new int[ItemCount];

            for (int i = 0; i < ItemCount; i++)
            {
                inversionTable[i] = 0;
                direction[i] = 1;
            }

            /* trivial case */
            if (ItemCount == 1)
            {
                VisitPermutation();
                return;
            }

            VisitPermutation();
            while (true)
            {
                // prepare for change
                int j = ItemCount - 1;
                int s = 0;

                bool goon = true;

                while (goon)
                {
                    // ready to change?
                    int q = inversionTable[j] + direction[j];
                    if (q < 0 || q == j + 1)
                    {
                        if (q == j + 1)
                        {
                            // increase s
                            if (j == 0)
                            {
                                return;
                            }
                            else
                            {
                                s++;
                            }
                        }

                        // switch direction
                        direction[j] = -direction[j];
                        j--;
                    }
                    else
                    {
                        // change
                        Swap(ref items[j - inversionTable[j] + s], ref items[j - q + s]);

                        inversionTable[j] = q;
                        VisitPermutation();
                        goon = false;

                    }
                }
            }
        }
    }
}
