﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PermutationAlgorithms.PermAlgorithms.Impl
{
    /// <summary>
    /// Plain change algorithm (Algorithm T)
    /// from:
    /// Donald E. Knuth - The Art Of Computer Programming Pre-Fascicle 2B.
    /// A Draft Of Section 7.2.1.2: Generating all permutations
    /// </summary>
    class AlgorithmT : AbstractPermAlgo
    {
        private int[] transitionTable;

        public AlgorithmT(int numOfItems, IPermutationVisitor permVisitor) :
            base(numOfItems, permVisitor)
        {
            GenerateTransitionTable();
        }

        private void GenerateTransitionTable()
        {
            bool t2 = true;
            bool t3 = true;

            // T1 Init
            long N = Factorial(ItemCount);
            long d = N / 2;
            transitionTable = new int[N];
            transitionTable[d] = 1;
            int m = 2;

            if (ItemCount == 1)
            {
                return;
            }

            while (t2)
            {
                // T2 Loop on m
                if (m == ItemCount)
                {
                    t2 = false;
                    continue;
                }

                m++;
                d = d / m;
                long k = 0;

                t3 = true;
                while (t3)
                {
                    // T3 hunt down
                    k += d;
                    int j = m - 1;
                    while (j > 0)
                    {
                        transitionTable[k] = j;
                        j--;
                        k += d;
                    }

                    // T4 offset
                    transitionTable[k]++;
                    k = k + d;

                    // T5 hunt up
                    while (j < m - 1)
                    {
                        j++;
                        transitionTable[k] = j;
                        k += d;
                    }

                    if (k < N)
                    {
                        t3 = true;
                    }
                    else
                    {
                        t3 = false;
                    }
                }
            }
        }

        public override void Generate()
        {
            //PrintTable(transitionTable);
            PrintPermutations(transitionTable, ItemCount);
        }

        private static void PrintTable(int[] t)
        {
            for (int i = 1; i < t.Length; i++)
            {
                Console.Out.Write(t[i] + " ");
            }
            Console.Out.WriteLine();
            Console.Out.Flush();
        }

        private void PrintPermutations(int[] t, int n)
        {
            InitItemSet();

            //PrintArray(values);
            VisitPermutation();
            for (int i = 1; i < t.Length; i++)
            {
                int index = t[i];

                Swap(ref items[index - 1], ref items[index]);

                VisitPermutation();
            }
        }
    }
}
